require 'tmail'
require 'net/smtp'
require 'forwardable'

module Iowa
	# A specialized component that lets one easily create and send customize
	# email.  An instance of Iowa::Email can send itself, or can return a text
	# representation suitable for piping into some other sending system.
	class Email < Component
		extend Forwardable
		def_delegators('@mail_obj','to','to=','from','from=','subject','subject=','date','date=','mime_version','mime_version=','content_type','content_type=','body','body=','cc','cc=','bcc','bcc=','reply_to','reply_to=','sender','sender=','message_id','message_id=','in_reply_to','in_reply_to=','references','references=','transfer_encoding','transfer_encoding=','disposition','disposition=')

		attr_accessor :mail_obj, :smtp_account, :smtp_password, :smtp_connection

		def initialize(name, bindings, attributes, parent)
			super(name,bindings,attributes,parent)
			email_init
		end

		def email_init(params = {})
			@body_generated_flag = false
			@mail_obj = TMail::Mail.new unless mail_obj
			self.subject = " "
			self.content_type = 'text/plain'
			if h = self.class.instance_variable_get('@additional_headers')
				h.each do |k,v|
					@mail_obj[k] = v
				end
			end
			[:smtp_server, :smtp_port, :helo_domain, :authentication, :smtp_account, :smtp_password, :recipients, :to, :from, :subject, :date, :sent_on, :mime_version, :content_type, :body, :cc, :bcc, :reply_to, :sender, :message_id, :in_reply_to, :references, :transfer_encoding, :disposition].each do |param|
				if p = self.class.instance_variable_get("@#{param}")
					self.__send__("#{param}=", p)
				end
				self.__send__("#{param}=", params[param])if params.has_key?(param)
				params.delete(param)
			end
			params.each {|k,v| self[k] = v}
		end
		
		def body
			generate_body unless body_generated?
			mail_obj.body
		end
		
		def preamble
			generate_body unless body_generated?
			mail_obj.preamble
		end
			
		def epilogue
			generate_body unless body_generated?
			mail_obj.epilogue
		end
				
		def multipart?
			generate_body unless body_generated?
			mail_obj.multipart?
		end
			
		def body_generated?
			@body_generated_flag
		end
		
		def recipients
			mail_obj.to
		end
		
		def recipients=(recip_list)
			mail_obj.to = recip_list
		end
		
		def smtp_server
			@smtp_server.nil? ? "smtp.#{Socket.gethostname}" : @smtp_server
		end
		
		def smtp_server=(server)
			@smtp_server = server
		end
		
		def smtp_port
			@smtp_port.nil? ? 25 : @smtp_port
		end
		
		def smtp_port=(port)
			@smtp_port = port
		end
		
		def helo_domain
			@helo_domain.nil? ? Socket.gethostname : @helo_comain
		end
		
		def helo_domain=(domain)
			@helo_domain = domain
		end
		
		def authentication
			@authentication
		end
		
		def authentication=(value)
			@authentication = case value.to_s
				when 'plain'
					:plain
				when 'login'
					:login
				when 'cram_md5'
					:cram_md5
				else
					@authentication
			end
		end

		def sent_on
			date
		end
		
		def sent_on=(value)
			self.date = value
		end
		
		def [](key)
			mail_obj[key]
		end
		
		def []=(key,value)
			mail_obj[key] = value
		end
		
		def generate_body
			@mail_obj.each_pair {|h,v| File.open('/tmp/h.out','a+') {|fh| fh.puts "    :#{h} -- #{v}"}}
			File.open('/tmp/h.out','a+') {|fh| fh.puts "#  :to -- #{@mail_obj.to}"}
			@body_generated_flag = true
			email_context = Iowa::Context.new(session.context.request,'')
			email_context.sessionID = session.context.sessionID
            email_context.requestID = session.requestCount
			handleResponse(email_context)
			self.body = email_context.response
		end
		
		def encoded
			mail_obj.encoded
		end
		
		def send
			generate_body unless body_generated?
			if smtp_connection
				smtp_connection.send_message(mail_obj.encoded, from, recipients)
			else
				args = []
				args.push smtp_account if smtp_account
				args.push smtp_password if smtp_password
				args.push authentication if authentication
				Net::SMTP.start(smtp_server, smtp_port, helo_domain, *args) {|smtp|
					smtp.send_message(mail_obj.encoded, from, recipients)
				}
			end
		end
		alias :send_email :send

	end
end