require 'iowa'
require 'iowa/HTTPServer'
require 'rbconfig'

module Iowa
	def Iowa.checkConfiguration
		if @@config.has_key? 'logging'
			if @@config['logging'].has_key? 'basedir'
				raise "The logging basedir (#{@@config['logging']['basedir']}) must be a directory." unless FileTest.directory? @@config['logging']['basedir']
				raise "The logging basedir #{@@config['logging']['basedir']}) must be writeable." unless FileTest.writable? @@config['logging']['basedir']
			end
		
			raise "The minimum logging level (minlevel) must be an integer between 0 to 7, inclusively." if @@config['logging'].has_key? 'minlevel' and (@@config['logging']['minlevel'] < 0 or @@config['logging']['minlevel'] > 7)

			raise "The maximum log file size (maxsize) must be greater than zero." if @@config.has_key? 'maxsize' and @@config['maxsize'] <= 0

			raise "The maximum log file age (maxage) must be greater than zero." if @@config.has_key? 'maxage' and @@config['maxage'] <= 0
		end

		# Now apply any defaults for optional items which were not specified.

		@@config['logging'] = Hash.new unless @@config.has_key? 'logging'
		@@config['logging']['basedir'] = '/tmp' unless @@config['logging'].has_key? 'basedir'
		@@config['logging']['minlevel'] = 'info' unless @@config['logging'].has_key? 'minlevel'
		@@config['logging']['maxsize'] = 1024 * 1024 unless @@config['logging'].has_key? 'maxsize'
		@@config['logging']['maxage'] = 86400 unless @@config['logging'].has_key? 'maxage'
		if @@config['socket'].has_key? 'path'
			@@config['logging']['filename'] = "#{@@config['logging']['basedir']}/#{@@config['socket']['basename']}.log"
		else
			@@config['logging']['filename'] = "#{@@config['logging']['basedir']}/#{@@config['socket']['hostname']}:#{@@config['socket']['port']}.log"
		end
		
		@@config['webrick'] ||= {}
		@@config['webrick']['port'] ||= 2000
		@@config['webrick']['docroot'] ||= '../htdocs'
		@@config['webrick']['cgiprefix'] ||= '/cgi-bin'
		@@config['webrick']['cgiroot'] ||= '../cgi-bin'
	end

	def Iowa.run()
		mylog = Logger['iowa_log']

		ruby = File.join(::Config::CONFIG['bindir'],
			::Config::CONFIG['ruby_install_name'])
		ruby << ::Config::CONFIG['EXEEXT']

		@@config['webrick']['docroot'] = nil unless FileTest.exist?(@@config['webrick']['docroot'])
		
		mylog.info("Creating WEBrick server on port #{@@config['webrick']['port']} with a document root of #{@@config['webrick']['docroot']}.  The mapfile is at #{Iowa.app.mapfile}, and the Ruby interpreter used for CGI programs is #{ruby}.")
		server = Iowa::HTTPServer.new(
			:Port => @@config['webrick']['port'],
			:DocumentRoot => @@config['webrick']['docroot'],
			:MapFile => Iowa.app.mapfile,
			:Logger => mylog,
			:CGIInterpreter => ruby)
		
		if FileTest.exist?(@@config['webrick']['cgiroot'])
			file_handler_options = {:HandlerTable => Hash.new(WEBrick::HTTPServlet::CGIHandler)}
			mylog.info("Mounting #{@@config['webrick']['cgiroot']} as the cgi-bin directory at #{@@config['webrick']['cgiprefix']}.") 
			server.mount(@@config['webrick']['cgiprefix'], WEBrick::HTTPServlet::FileHandler, @@config['webrick']['cgiroot'], file_handler_options)
		end
				
		trap("INT") {server.shutdown}
		mylog.info("Starting WEBrick...")
		server.start
	end
end
